@@ -5,6 +5,7 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"io"
 	"os"
 	"os/exec"
 	"os/signal"
@@ -32,6 +33,7 @@ const (
 
 type shellOpts struct {
 	clear, background bool
+	pipes             []string
 	binary            string
 	banner            string
 	args              []string
@@ -96,40 +98,41 @@ func execute(opts shellOpts) error {
 	}
 	ctx, cancel := context.WithCancel(context.Background())
 	defer func() {
-		cancel()
-		clearScreen()
+		if !opts.background {
+			cancel()
+			clearScreen()
+		}
 	}()
 
 	sigChan := make(chan os.Signal, 1)
 	signal.Notify(sigChan, os.Interrupt, syscall.SIGTERM)
-	go func() {
+	go func(cancel context.CancelFunc) {
+		defer log.Debug().Msgf("SIGNAL_GOR - BAILED!!")
 		select {
 		case <-sigChan:
-			log.Debug().Msg("Command canceled with signal!")
+			log.Debug().Msgf("Command canceled with signal!")
 			cancel()
 		case <-ctx.Done():
-			return
+			log.Debug().Msgf("SIGNAL Context CANCELED!")
 		}
-	}()
+	}(cancel)
 
-	log.Debug().Msgf("Running command> %s %s", opts.binary, strings.Join(opts.args, " "))
-	cmd := exec.Command(opts.binary, opts.args...)
+	cmds := make([]*exec.Cmd, 0, 1)
+	cmd := exec.CommandContext(ctx, opts.binary, opts.args...)
+	log.Debug().Msgf("RUNNING> %s", cmd)
+	cmds = append(cmds, cmd)
 
-	var err error
-	if opts.background {
-		err = cmd.Start()
-	} else {
-		cmd.Stdin, cmd.Stdout, cmd.Stderr = os.Stdin, os.Stdout, os.Stderr
-		_, _ = cmd.Stdout.Write([]byte(opts.banner))
-		err = cmd.Run()
+	for _, p := range opts.pipes {
+		tokens := strings.Split(p, " ")
+		if len(tokens) < 2 {
+			continue
+		}
+		cmd := exec.CommandContext(ctx, tokens[0], tokens[1:]...)
+		log.Debug().Msgf("\t| %s", cmd)
+		cmds = append(cmds, cmd)
 	}
 
-	select {
-	case <-ctx.Done():
-		return errors.New("canceled by operator")
-	default:
-		return err
-	}
+	return pipe(ctx, opts, cmds...)
 }
 
 func runKu(a *App, opts shellOpts) (string, error) {
@@ -341,6 +344,11 @@ func k9sShellPod(node string, cfg *config.ShellPod) v1.Pod {
 				},
 			},
 			Containers: []v1.Container{c},
+			Tolerations: []v1.Toleration{
+				{
+					Operator: v1.TolerationOperator("Exists"),
+				},
+			},
 		},
 	}
 }
@@ -353,3 +361,58 @@ func asResource(r config.Limits) v1.ResourceRequirements {
 		},
 	}
 }
+
+func pipe(ctx context.Context, opts shellOpts, cmds ...*exec.Cmd) error {
+	if len(cmds) == 0 {
+		return nil
+	}
+
+	if len(cmds) == 1 {
+		cmd := cmds[0]
+		if opts.background {
+			cmd.Stdin, cmd.Stdout, cmd.Stderr = os.Stdin, log.Logger, log.Logger
+			return cmd.Start()
+		}
+		cmd.Stdin, cmd.Stdout, cmd.Stderr = os.Stdin, os.Stdout, os.Stderr
+		//cmd.SysProcAttr = &syscall.SysProcAttr{
+		////	//Setpgid:    true,
+		////	//Setctty:    true,
+		//	Foreground: true,
+		//}
+		_, _ = cmd.Stdout.Write([]byte(opts.banner))
+
+		log.Debug().Msgf("Running Start")
+		err := cmd.Run()
+		log.Debug().Msgf("Running Done")
+		return err
+		// select {
+		// case <-ctx.Done():
+		// 	return errors.New("canceled by operator")
+		// default:
+		// 	log.Debug().Msgf("PIPE RETURN %s", err)
+		// 	return err
+		// }
+	}
+
+	last := len(cmds) - 1
+	for i := 0; i < len(cmds); i++ {
+		cmds[i].Stderr = os.Stderr
+		if i+1 < len(cmds) {
+			r, w := io.Pipe()
+			cmds[i].Stdout, cmds[i+1].Stdin = w, r
+		}
+	}
+	cmds[last].Stdout = os.Stdout
+
+	for _, cmd := range cmds {
+		log.Debug().Msgf("Starting CMD %s", cmd)
+		if err := cmd.Start(); err != nil {
+			return err
+		}
+	}
+
+	log.Debug().Msgf("WAITING!!!")
+	err := cmds[len(cmds)-1].Wait()
+	log.Debug().Msgf("DONE WAITING!!!")
+	return err
+}
