@@ -2,22 +2,25 @@ package view
 
 import (
 	"fmt"
+	"math"
 	"regexp"
 	"strings"
 
 	"github.com/derailed/k9s/internal/client"
+	"github.com/derailed/k9s/internal/port"
 	"github.com/derailed/k9s/internal/render"
 	"github.com/derailed/k9s/internal/ui"
 	"github.com/derailed/tview"
+	"github.com/rs/zerolog/log"
 )
 
 const portForwardKey = "portforward"
 
 // PortForwardCB represents a port-forward callback function.
-type PortForwardCB func(v ResourceViewer, path, co string, mapper []client.PortTunnel)
+type PortForwardCB func(ResourceViewer, string, port.PortTunnels) error
 
 // ShowPortForwards pops a port forwarding configuration dialog.
-func ShowPortForwards(v ResourceViewer, path string, ports []string, okFn PortForwardCB) {
+func ShowPortForwards(v ResourceViewer, path string, ports port.ContainerPortSpecs, aa port.Annotations, okFn PortForwardCB) {
 	styles := v.App().Styles.Dialog()
 
 	f := tview.NewForm()
@@ -31,18 +34,29 @@ func ShowPortForwards(v ResourceViewer, path string, ports []string, okFn PortFo
 
 	address := v.App().Config.CurrentCluster().PortForwardAddress
 
-	var p1, p2 string
-	if len(ports) > 0 {
-		p1, p2 = ports[0], extractPort(ports[0])
+	pf, err := aa.PreferredPorts(ports)
+	if err != nil {
+		log.Warn().Err(err).Msgf("unable to resolve ports")
 	}
 
-	f.AddInputField("Container Port:", p1, 30, nil, func(p string) {
-		p1 = p
+	p1, p2 := pf.ToPortSpec(ports)
+	fieldLen := int(math.Max(30, float64(len(p1))))
+	f.AddInputField("Container Port:", p1, fieldLen, nil, nil)
+	coField := f.GetFormItemByLabel("Container Port:").(*tview.InputField)
+	if coField.GetText() == "" {
+		coField.SetPlaceholder("Enter a container name/port")
+	}
+	f.AddInputField("Local Port:", p2, fieldLen, nil, nil)
+	poField := f.GetFormItemByLabel("Local Port:").(*tview.InputField)
+	if poField.GetText() == "" {
+		poField.SetPlaceholder("Enter a local port")
+	}
+	coField.SetChangedFunc(func(s string) {
+		port := extractPort(s)
+		poField.SetText(port)
+		p2 = port
 	})
-	f.AddInputField("Local Port:", p2, 30, nil, func(p string) {
-		p2 = p
-	})
-	f.AddInputField("Address:", address, 30, nil, func(h string) {
+	f.AddInputField("Address:", address, fieldLen, nil, func(h string) {
 		address = h
 	})
 	for i := 0; i < 3; i++ {
@@ -55,21 +69,18 @@ func ShowPortForwards(v ResourceViewer, path string, ports []string, okFn PortFo
 	}
 
 	f.AddButton("OK", func() {
-		pp1 := strings.Split(p1, ",")
-		pp2 := strings.Split(p2, ",")
-		if len(pp1) == 0 || len(pp1) != len(pp2) {
+		if coField.GetText() == "" || poField.GetText() == "" {
 			v.App().Flash().Err(fmt.Errorf("container to local port mismatch"))
 			return
 		}
-		var tt []client.PortTunnel
-		for i := range pp1 {
-			tt = append(tt, client.PortTunnel{
-				Address:       address,
-				LocalPort:     pp2[i],
-				ContainerPort: extractPort(pp1[i]),
-			})
+		tt, err := port.ToTunnels(address, coField.GetText(), poField.GetText())
+		if err != nil {
+			v.App().Flash().Err(err)
+			return
+		}
+		if err := okFn(v, path, tt); err != nil {
+			v.App().Flash().Err(err)
 		}
-		okFn(v, path, extractContainer(pp1[0]), tt)
 	})
 	pages := v.App().Content.Pages
 	f.AddButton("Cancel", func() {
@@ -84,12 +95,12 @@ func ShowPortForwards(v ResourceViewer, path string, ports []string, okFn PortFo
 		b.SetLabelColorActivated(styles.ButtonFocusFgColor.Color())
 	}
 
-	modal := tview.NewModalForm(fmt.Sprintf("<PortForward on %s>", path), f)
-
-	if len(ports) != 0 {
-		modal.SetText("Exposed Ports: " + strings.Join(ports, ","))
+	modal := tview.NewModalForm("<PortForward>", f)
+	msg := path
+	if len(ports) > 1 {
+		msg += "\n\nExposed Ports:\n" + ports.Dump()
 	}
-
+	modal.SetText(msg)
 	modal.SetTextColor(styles.FgColor.Color())
 	modal.SetBackgroundColor(styles.BgColor.Color())
 	modal.SetDoneFunc(func(_ int, b string) {
